from langchain.text_splitter import CharacterTextSplitter
import re
from typing import List


class AliTextSplitter(CharacterTextSplitter):
    def __init__(self, pdf: bool = False, **kwargs):
        super().__init__(**kwargs)
        self.pdf = pdf

    def split_text(self, text: str) -> List[str]:
        # use_document_segmentation参数指定是否用语义切分文档，此处采取的文档语义分割模型为达摩院开源的nlp_bert_document-segmentation_chinese-base，论文见https://arxiv.org/abs/2107.09278
        # 如果使用模型进行文档语义切分，那么需要安装modelscope[nlp]：pip install "modelscope[nlp]" -f https://modelscope.oss-cn-beijing.aliyuncs.com/releases/repo.html
        # 考虑到使用了三个模型，可能对于低配置gpu不太友好，因此这里将模型load进cpu计算，有需要的话可以替换device为自己的显卡id
        if self.pdf:
            text = re.sub(r"\n{3,}", r"\n", text)
            text = re.sub('\s', " ", text)
            text = re.sub("\n\n", "", text)
        from modelscope.pipelines import pipeline

        p = pipeline(
            task="document-segmentation",
            model='damo/nlp_bert_document-segmentation_chinese-base',
            device="cpu")
        result = p(documents=text)
        sent_list = [i for i in result["text"].split("\n\t") if i]
        return sent_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : common
# @Time         : 2023/5/29 13:16
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *


def article_generator(keywords, api_key, max_tokens=2048):
    import openai
    openai.api_base = 'https://api.openai-proxy.com/v1'
    openai.api_key = api_key
    prompt = """
        你现在是位资深SEO优化专家，请通过尖括号里的关键词<{keywords}>，写一篇seo文章，
        要求返回标题与内容
        """.strip()
    model = "text-davinci-003"
    completion = openai.Completion.create(model=model, prompt=prompt, stream=True, max_tokens=max_tokens)
    for c in tqdm(completion, desc=keywords):
        yield c.choices[0].text



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : barkup
# @Time         : 2024/1/9 09:50
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *

from openai import OpenAI
from chatllm.schemas.openai_types import chat_completion_error, chat_completion_chunk_error, completion_keys


def create(**data):
    data = {key: data.get(key) for key in completion_keys if key in data}
    data['model'] = "backup-gpt" if data['model'].startswith('backup-gpt-4') else "backup-gpt-4"

    try:
        _ = (
            OpenAI(
                api_key=os.getenv("BACKUP_API_KEY"),
                base_url=os.getenv("OPENAI_BASE_URL", "https://api.chatllm.vip/v1")
            )
            .chat.completions.create(**data)
        )
        # logger.debug(_)
        return _
    except Exception as e:
        if data.get('stream'):
            chat_completion_chunk_error.choices[0].delta.content = str(e)
            return chat_completion_chunk_error
        else:
            chat_completion_error.choices[0].message.content = str(e)
            return chat_completion_error


if __name__ == '__main__':
    data = {
        'model': 'gpt-3.5-turbo',
        'messages': [
            {'role': 'system', 'content': "你是gpt4, Let's think things through one step at a time."},
            {'role': 'user', 'content': '1+1'}
        ],
        'stream': False,
        'xxx': 111
    }

    print(create(**data))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : faiss
# @Time         : 2023/8/10 16:21
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 比较 https://zhuanlan.zhihu.com/p/595249861?utm_id=0

from meutils.pipe import *
from langchain.vectorstores import FAISS as _FAISS
from langchain.embeddings.base import Embeddings
from langchain.docstore.base import AddableMixin, Docstore
from langchain.docstore.document import Document
from langchain.embeddings.base import Embeddings
from langchain.vectorstores.utils import DistanceStrategy, maximal_marginal_relevance


class FAISS(_FAISS):
    """
    from chatllm.llmchain.vectorstores import FAISS
    from chatllm.llmchain.embeddings import HuggingFaceEmbeddings

    model_name = '/Users/betterme/PycharmProjects/AI/m3e-small'
    encode_kwargs = {'normalize_embeddings': True}
    embeddings = HuggingFaceEmbeddings(model_name=model_name, encode_kwargs=encode_kwargs)

    faiss1 = FAISS.from_texts(['1', '2', '3'], embeddings)
    faiss2 = FAISS.from_texts(['11', '22', '33'], embeddings)
    faiss1.merge_from(faiss2)
    faiss1.similarity_search('1')

    faiss1.save_local('faiss_data')
    faiss1 = FAISS.load_local('faiss_data', embeddings)

    """

    def __init__(
        self,
        embedding_function: Callable,
        index: Any,
        docstore: Docstore,
        index_to_docstore_id: Dict[int, str],
        relevance_score_fn: Optional[Callable[[float], float]] = None,
        normalize_L2: bool = False,
        distance_strategy: DistanceStrategy = DistanceStrategy.MAX_INNER_PRODUCT,
        **kwargs
    ):
        super().__init__(embedding_function, index, docstore, index_to_docstore_id, relevance_score_fn, normalize_L2,
                         distance_strategy)

    @classmethod
    def from_texts(
        cls,
        texts: List[str],
        embedding: Embeddings,
        metadatas: Optional[List[dict]] = None,
        ids: Optional[List[str]] = None,
        **kwargs: Any,
    ) -> _FAISS:

        # 默认内积
        kwargs['distance_strategy'] = kwargs.get('distance_strategy', DistanceStrategy.MAX_INNER_PRODUCT)

        return super().from_texts(texts, embedding, metadatas, ids, **kwargs)

    def similarity_search(
        self,
        query: str,
        k: int = 4,
        filter: Optional[Dict[str, Any]] = None,
        fetch_k: int = 20,
        threshold: float = 0.5,
        **kwargs: Any,
    ) -> List[Document]:
        """Return docs most similar to query.
        faiss.as_retriever(search_kwargs={'k':1})

        Args:
            query: Text to look up documents similar to.
            k: Number of Documents to return. Defaults to 4.
            filter: (Optional[Dict[str, str]]): Filter by metadata. Defaults to None.
            fetch_k: (Optional[int]) Number of Documents to fetch before filtering.
                      Defaults to 20.
            threshold:

        Returns:
            List of Documents most similar to the query.
        """

        docs_and_scores = self.similarity_search_with_score(
            query, k, filter=filter, fetch_k=fetch_k, **kwargs
        )

        docs = []
        for doc, score in docs_and_scores:
            if score > threshold:
                doc.metadata['score'] = round(score, 2)
                docs.append(doc)
        return docs


if __name__ == '__main__':
    from chatllm.llmchain.vectorstores import FAISS
    from chatllm.llmchain.embeddings import HuggingFaceEmbeddings

    model_name = '/Users/betterme/PycharmProjects/AI/embeddings/m3e-small'
    encode_kwargs = {'normalize_embeddings': True, 'show_progress_bar': True, 'device': 'mps'}
    embeddings = HuggingFaceEmbeddings(model_name=model_name, encode_kwargs=encode_kwargs)

    faiss1 = FAISS.from_texts(['1', '2', '3'], embeddings)
    faiss2 = FAISS.from_texts(['11', '22', '33'], embeddings)
    faiss1.merge_from(faiss2)
    print(faiss1.similarity_search('1'))

    # faiss1.save_local('faiss_data')
    # faiss1 = FAISS.load_local('faiss_data', embeddings)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chatmind
# @Time         : 2023/5/4 11:46
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://mp.weixin.qq.com/s/gbqd8bzKRbbOqf5ebHICWg

from chatllm.llms import load_llm4chat
from meutils.pipe import *


class ChatMind(object):

    def __init__(self):
        role = """
        请扮演为一个思维导图制作者，必须使用中文回答。
        要求：
        1. 根据《》中的主题创作
        2. 输出格式为markdown: "# "表示中央主题， "## "表示主要主题，"### "表示子主题，"- "表示叶子节点"
        3. 包含多个主题和子主题，以及叶子节点。
        4. 叶子节点内容长度10-50
        """
        self.history = [
            {"role": "system", "content": role},
        ]

    def __call__(self, **kwargs):
        return self.qa(**kwargs)

    def qa(self, title, **kwargs):
        return self.do_chat(f"《{title}》", history=self.history)

    def load_llm(self, model_name_or_path="chatgpt", **kwargs):
        self.do_chat = load_llm4chat(model_name_or_path, **kwargs)

    def set_chat_kwargs(self, **kwargs):
        self.do_chat = partial(self.do_chat, **kwargs)

    def mind_html(self, md='# Title\n ## SubTitle\n - Node'):
        # 后处理
        md = re.sub(r'(?m)^(?![\-#]).+', '\n', md) # 仅保留 # - 开头

        from jinja2 import Template, Environment, PackageLoader, FileSystemLoader

        env = Environment(loader=PackageLoader('meutils'))
        template = env.get_template('markmap.html')
        return template.render(md=md)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : cocopilot
# @Time         : 2023/12/6 12:17
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import os

from meutils.pipe import *

import requests
from flask import Flask, request, Response, jsonify
import uuid
import datetime
import hashlib

app = Flask(__name__)

machine_id = hashlib.sha256(str(uuid.uuid4()).encode()).hexdigest()

GHO_TOKEN = os.getenv("GHO_TOKEN")


def forward_request(GHO_TOKEN: str = GHO_TOKEN, json_data=None):
    headers = {
        'Host': 'api.github.com',
        'authorization': f'token {GHO_TOKEN}',
        "Editor-Version": "vscode/1.84.2",
        "Editor-Plugin-Version": "copilot/1.138.0",
        "User-Agent": "GithubCopilot/1.138.0",
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate, br",
        "Connection": "close"
    }

    response = requests.get(
        'https://api.github.com/copilot_internal/v2/token', headers=headers)
    print("Auth:", response.text)
    if response.status_code == 200 and response.json():
        access_token = response.json()['token']

        acc_headers = {
            'Authorization': f'Bearer {access_token}',
            'X-Request-Id': str(uuid.uuid4()),
            'Vscode-Sessionid': str(uuid.uuid4()) + str(int(datetime.datetime.utcnow().timestamp() * 1000)),
            'vscode-machineid': machine_id,
            'Editor-Version': 'vscode/1.84.2',
            'Editor-Plugin-Version': 'copilot-chat/0.10.2',
            'Openai-Organization': 'github-copilot',
            'Openai-Intent': 'conversation-panel',
            'Content-Type': 'application/json',
            'User-Agent': 'GitHubCopilotChat/0.10.2',
            'Accept': '*/*',
            'Accept-Encoding': 'gzip, deflate, br',
        }

        resp = requests.post('https://api.githubcopilot.com/chat/completions', headers=acc_headers, json=json_data)
        return resp


@app.route('/v1/chat/completions', methods=['POST'])
def proxy():
    # 从请求中获取json数据
    json_data = request.get_json()
    if json_data is None:
        return "Request body is missing or not in JSON format", 400
    # 获取Authorization头部信息
    GHO_TOKEN = request.headers.get('Authorization')
    GHO_TOKEN = GHO_TOKEN.split(' ')[1]
    print("Secret:", GHO_TOKEN)
    print("Message:", json_data)

    # Check if stream option is set in the request data
    stream = json_data.get('stream', False)

    # 转发请求并获取响应
    resp = forward_request(json_data=json_data)
    # 处理流式输出

    if stream:
        return Response(generate_chunks(resp), content_type='application/json')

    return Response(resp.content, content_type='application/json')


def generate_chunks(response):
    for chunk in response.iter_content(chunk_size=8192):
        yield chunk.decode('utf-8')


@app.route('/v1/models', methods=['GET'])
def models():
    data = {
        "object": "list",
        "data": [
            {"id": "gpt-4-0314", "object": "model", "created": 1687882410,
             "owned_by": "openai", "root": "gpt-4-0314", "parent": None},
            {"id": "gpt-4-0613", "object": "model", "created": 1686588896,
             "owned_by": "openai", "root": "gpt-4-0613", "parent": None},
            {"id": "gpt-4", "object": "model", "created": 1687882411,
             "owned_by": "openai", "root": "gpt-4", "parent": None},
            {"id": "gpt-3.5-turbo", "object": "model", "created": 1677610602,
             "owned_by": "openai", "root": "gpt-3.5-turbo", "parent": None},
            {"id": "gpt-3.5-turbo-0301", "object": "model", "created": 1677649963,
             "owned_by": "openai", "root": "gpt-3.5-turbo-0301", "parent": None},
        ]
    }
    return jsonify(data)


# GHO_TOKEN = "gho_xx"
# set_access_token(get_token(GHO_TOKEN)['token'])

if __name__ == '__main__':
    app.run(host='0.0.0.0', port=8080, debug=False)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : copilot
# @Time         : 2023/12/6 13:14
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 不准白嫖 必须 star, todo: 兜底设计、gpt/图片

from meutils.pipe import *
from meutils.cache_utils import ttl_cache
from meutils.decorators.retry import retrying
from meutils.queues.uniform_queue import UniformQueue
from meutils.async_utils import sync_to_async
from meutils.notice.feishu import send_message

from openai.types.chat.chat_completion import ChatCompletion
from openai.types.chat.chat_completion_chunk import ChatCompletionChunk
from chatllm.schemas.openai_types import chat_completion_error, chat_completion_chunk_error
from chatllm.llmchain.completions import backup_completions

requests.post = retrying(requests.post)

GITHUB_BASE_URL = os.getenv('GITHUB_BASE_URL', 'https://api.github.com')
GITHUB_COPILOT_BASE_URL = os.getenv('GITHUB_COPILOT_BASE_URL', 'https://api.githubcopilot.com')

send_message = partial(
    send_message,
    title="Kimi",
    url="https://open.feishu.cn/open-apis/bot/v2/hook/e2f5c8eb-4421-4a0b-88ea-e2d9441990f2"
)


class Completions(object):
    def __init__(self, **client_params):
        self.api_key = client_params.get('api_key')
        self.access_token = self.get_access_token(self.api_key)

    def create(self, messages: Union[str, List[Dict[str, Any]]], **kwargs):  # ChatCompletionRequest: 定义请求体

        data = {
            "model": 'gpt-4',
            "messages": messages if isinstance(messages, list) else [{"role": "user", "content": messages}],
            **kwargs
        }

        # logger.debug(data)

        if data.get('stream'):
            # interval = data.get('interval', 0.05)
            interval = 0.05 if "gpt-4" in data['model'] else 0.01
            return self.smooth_stream(interval=interval, **data)
        else:
            return self._create(**data)

    def create_sse(self, **data):
        response = self.create(**data)
        if data.get('stream'):
            from sse_starlette import EventSourceResponse
            generator = (chunk.model_dump_json() for chunk in response)
            return EventSourceResponse(generator, ping=10000)
        return response

    @sync_to_async(thread_sensitive=False)
    def acreate(self, messages: Union[str, List[Dict[str, Any]]], **kwargs):
        """
            generator = (chunk.model_dump_json() for chunk in completions.acreate(messages)
        """
        return self.create(messages, **kwargs)

    def _create(self, **data):
        response = self._post(**data)
        if response.status_code != 200:
            if response.text.strip() == "Unprocessable Entity":
                return backup_completions.create(**data)  # 兜底

            chat_completion_error.choices[0].message.content = response.text
            return chat_completion_error

        response = response.json()
        response['model'] = data.get('model', 'gpt-4')
        response['object'] = 'chat.completion'
        response['choices'][0]['logprobs'] = None
        completion = ChatCompletion.model_validate(response)

        return completion

    def _stream_create(self, **data):
        response = self._post(**data)
        # logger.debug(response.text)

        if response.status_code != 200:
            chat_completion_chunk_error.choices[0].delta.content = response.text  # 流式好像不会出现 Unprocessable Entity
            yield chat_completion_chunk_error
            return

        for chunk in response.iter_lines(chunk_size=1024):
            if chunk and chunk != b'data: [DONE]':
                # logger.debug(chunk)

                chunk = chunk.strip(b"data: ")
                chunk = json.loads(chunk)
                chunk['model'] = data.get('model', "gpt-4")
                chunk['object'] = "chat.completion.chunk"
                chunk['choices'][0]['finish_reason'] = chunk['choices'][0].get('finish_reason')  # 最后为 "stop"
                chunk = ChatCompletionChunk.model_validate(chunk)

                chunk.choices[0].delta.role = 'assistant'
                content = chunk.choices[0].delta.content or ''
                chunk.choices[0].delta.content = content

                if content or chunk.choices[0].finish_reason:
                    # logger.debug(chunk)

                    if chunk.choices[0].finish_reason == "content_filter":
                        yield from backup_completions.create(**data)

                        # 告警
                        title = "OPENAI: CONTENT_FILTER"
                        send_message(title=title, content=chunk.model_dump_json())

                    else:
                        yield chunk

    def _post(self, **data):
        headers = {
            'Authorization': f'Bearer {self.access_token}',
            # 'X-Request-Id': str(uuid.uuid4()),
            # 'Vscode-Sessionid': str(uuid.uuid4()) + str(int(datetime.datetime.utcnow().timestamp() * 1000)),
            # 'vscode-machineid': machine_id,
            'Editor-Version': 'vscode/1.84.2',
            'Editor-Plugin-Version': 'copilot-chat/0.10.2',
            'Openai-Organization': 'github-copilot',
            'Openai-Intent': 'conversation-panel',
            'Content-Type': 'application/json',
            'User-Agent': 'GitHubCopilotChat/0.10.2',
            'Accept': '*/*',
            'Accept-Encoding': 'gzip, deflate, br',
        }
        url: str = f"{GITHUB_COPILOT_BASE_URL}/chat/completions"
        response = requests.post(
            url,
            json=data,
            headers=headers,
            stream=data.get('stream')
        )
        # logger.debug(response.status_code) # 400 # bad request: Authorization header is badly formatted

        if response.status_code != 200:  # todo: 兜底
            send_message(title=self.__class__.__name__, content=f"{response.text}\n\n{self.api_key}")

        return response

    @staticmethod
    @retrying
    @ttl_cache(ttl=15 * 60)  # 1500
    def get_access_token(api_key: Optional[str] = None):
        """
        {
        'annotations_enabled': True,
        'chat_enabled': True,
        'chat_jetbrains_enabled': False,
        'code_quote_enabled': True,
        'copilot_ide_agent_chat_gpt4_small_prompt': False,
        'copilotignore_enabled': False,
        'expires_at': 1703653893,
        'intellij_editor_fetcher': False,
        'prompt_8k': True,
        'public_suggestions': 'disabled',
        'refresh_in': 1500,
        'sku': 'yearly_subscriber',
        'snippy_load_test_enabled': False,
        'telemetry': 'enabled',
        'token': 'tid=925df46f15a3245bfa77d9f47cc073e1;exp=1703653893;sku=yearly_subscriber;st=dotcom;chat=1;rt=1;8kp=1:d5aeff67b9daa42231871e2434ad17152c6e0a68f685fa630e1bae1148283964',
        'tracking_id': '925df46f15a3245bfa77d9f47cc073e1',
        'vsc_panel_v2': False
        }
        {
            "annotations_enabled": true,
            "chat_enabled": true,
            "chat_jetbrains_enabled": false,
            "code_quote_enabled": false,
            "copilot_ide_agent_chat_gpt4_small_prompt": false,
            "copilotignore_enabled": false,
            "expires_at": 1702022150,
            "prompt_8k": true,
            "public_suggestions": "enabled",
            "refresh_in": 1500,
            "sku": "free_educational",
            "snippy_load_test_enabled": false,
            "telemetry": "enabled",
            "token": "tid=74069a4394491f4f41fc74888f24a0ab;exp=1702022150;sku=free_educational;st=dotcom;chat=1;sn=1;rt=1;8kp=1:b78c67b8a5d886b71b13a956f378d9b955299e5ad156a5699ab12cbeb5e7b960",
            "tracking_id": "74069a4394491f4f41fc74888f24a0ab",
            "vsc_panel_v2": false
        }
        """
        api_key = api_key or os.getenv("GITHUB_COPILOT_TOKEN")
        assert api_key

        headers = {
            'Host': 'api.github.com',
            'authorization': f'token {api_key}',
            "Editor-Version": "vscode/1.84.2",
            "Editor-Plugin-Version": "copilot/1.138.0",
            "User-Agent": "GithubCopilot/1.138.0",
            "Accept": "*/*",
            "Accept-Encoding": "gzip, deflate, br",
            "Connection": "close"
        }
        url = f"{GITHUB_BASE_URL}/copilot_internal/v2/token"
        response = requests.get(url, headers=headers)

        return response.json().get('token')  # response.get('error_details') 监控到 token 失效，返回 error_details，半小时过期

    @staticmethod
    def break_fn(line: ChatCompletionChunk):
        return line.choices[0].finish_reason

    def smooth_stream(self, interval: Optional[float] = None, **data):
        stream = self._stream_create(**data)
        if interval:
            stream = UniformQueue(stream).consumer(interval=interval, break_fn=self.break_fn)
        return stream

    @classmethod
    def chat(cls, data: dict):  # TEST
        """
            Completions.chat(data)
        """
        with timer('聊天测试'):
            _ = cls().create(**data)

            print(f'{"-" * 88}\n')
            if isinstance(_, Generator):
                for i in _:
                    content = i.choices[0].delta.content
                    print(content, end='')
            else:
                print(_.choices[0].message.content)
            print(f'\n\n{"-" * 88}')


if __name__ == '__main__':
    # 触发风控
    s = """
    Question:已知节点类型只有六种：原因分析、排故方法、故障时间、故障现象、故障装备单位、训练地点，现在我给你一个问题，你需要根据这个句子来推理出这个问题的答案在哪个节点类型中，问题是”管道细长、阻力太大时的轴向柱塞泵故障如何解决？“,输出格式形为：["节点类型1"], ["节点类型2"], …。除了这个列表以外请不要输出别的多余的话。
['排故方法']

Question:已知节点类型只有六种：原因分析、排故方法、故障时间、故障现象、故障装备单位、训练地点，现在我给你一个问题，你需要根据这个句子来推理出这个问题的答案在哪个节点类型中，问题是”转向缸出现爬行现象，但是压力表却忽高忽低，相对应的解决方案是？“输出格式形为：["节点类型1"], ["节点类型2"], …。除了这个列表以外请不要输出别的多余的话。
['原因分析']、['排故方法']

Question:已知节点类型只有六种：原因分析、排故方法、故障时间、故障现象、故障装备单位、训练地点，现在我给你一个问题，你需要根据这个句子来推理出这个问题的答案在哪个节点类型中，问题是”在模拟训练场A，轴向柱塞马达出现过什么故障？“输出格式形为：["节点类型1"], ["节点类型2"], …。除了这个列表以外请不要输出别的多余的话。

['故障现象']

已知节点类型只有六种：原因分析、排故方法、故障时间、故障现象、故障装备单位、训练地点，现在我给你一个问题，你需要根据这个句子来推理出这个问题的答案在哪个节点类型中，问题是”密封圈挤出间隙的解决方法是什么？“。输出格式形为：["节点类型1"], ["节点类型2"], …。除了这个列表以外请不要输出别的多余的话。
    """

    # s = "讲个故事"
    # s = '树上9只鸟，打掉1只，还剩几只'

    data = {
        'model': 'gpt-4',
        'messages': [
            {'role': 'system', 'content': "你是gpt4, Let's think things through one step at a time."},
            {'role': 'user', 'content': s}
        ],
        'stream': True}

    Completions.chat(data)

    # async def main():
    #     _ = await Completions().acreate(**data)
    #
    #     content = ''
    #     for i in _:
    #         content += i.choices[0].delta.content
    #     return content
    #
    #
    # print(arun(main()))

    # with timer('异步'):
    #     print([Completions().acreate(**data) for _ in range(10)] | xAsyncio)

    # data = {
    #     'model': 'gpt-xxx',
    #     'messages': [{'role': 'user', 'content': '讲个故事。 要足够长，这对我很重要。'}],
    #     'stream': False,
    #     # 'max_tokens': 16000
    # }
    # data = {
    #     'model': 'gpt-4',
    #     'messages': '树上9只鸟，打掉1只，还剩几只',  # [{'role': 'user', 'content': '树上9只鸟，打掉1只，还剩几只'}],
    #     'stream': False,
    #     'temperature': 0,
    #     # 'max_tokens': 32000
    # }
    #
    # for i in tqdm(range(1000)):
    #     _ = Completions().create(**data)
    #     print(_.choices[0].message.content)
    #     break
